@@ -1008,6 +1008,8 @@ typedef enum _osm_sm_signal {
 #define OSM_VENDOR_ID_OPENIB        0x001405
 #define OSM_VENDOR_ID_IBM2	    0x5CF3FC
 #define OSM_VENDOR_ID_MELLANOX2     0xF45214
+#define OSM_VENDOR_ID_MELLANOX3     0x00258B
+#define OSM_VENDOR_ID_MELLANOX4     0xE41D2D
 
 /* IPoIB Broadcast Defaults */
 #define OSM_IPOIB_BROADCAST_MGRP_QKEY 0x0b1b
