@@ -2,6 +2,7 @@
  * Copyright (c) 2006-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2009 HNR Consulting. All rights reserved.
  * Copyright (c) 2012 Lawrence Livermore National Lab.  All rights reserved.
+ * Copyright (c) 2014 Mellanox Technologies LTD. All rights reserved.
  *
  * This software is available to you under a choice of one of two
  * licenses.  You may choose to be licensed under the terms of the GNU
@@ -173,7 +174,7 @@ static void cc_setup_mad_data(osm_sm_t * p_sm)
 
 	/* Congestion Control Table */
 
-	/* if no entries, we will always send atleast 1 mad to set ccti_limit = 0 */
+	/* if no entries, we will always send at least 1 mad to set ccti_limit = 0 */
 	if (!p_opt->cc_cct.entries_len)
 		p_cc->cc_tbl_mads = 1;
 	else {
@@ -440,7 +441,6 @@ static inline void decrement_outstanding_mads(osm_congestion_control_t *p_cc)
 	cl_event_signal(&p_cc->sig_mads_on_wire_continue);
 }
 
-
 static void cc_rcv_mad(void *context, void *data)
 {
 	osm_congestion_control_t *p_cc = context;
@@ -449,16 +449,19 @@ static void cc_rcv_mad(void *context, void *data)
 	ib_cc_mad_t *p_cc_mad;
 	osm_madw_context_t *p_mad_context = &p_madw->context;
 	ib_mad_t *p_mad = osm_madw_get_mad_ptr(p_madw);
-	uint64_t node_guid = p_mad_context->cc_context.node_guid;
-	uint64_t port_guid = p_mad_context->cc_context.port_guid;
+	ib_net64_t node_guid = p_mad_context->cc_context.node_guid;
+	ib_net64_t port_guid = p_mad_context->cc_context.port_guid;
 	uint8_t port = p_mad_context->cc_context.port;
 	osm_port_t *p_port;
 
 	OSM_LOG_ENTER(p_cc->log);
 
 	OSM_LOG(p_cc->log, OSM_LOG_VERBOSE,
-		"Processing received MAD status 0x%x context 0x%"
-		PRIx64 "port %u\n", p_mad->status, node_guid, port);
+		"Processing received MAD status 0x%x for "
+		"attr ID %u mod 0x%x node 0x%" PRIx64 " port %u\n",
+		cl_ntoh16(p_mad->status), cl_ntoh16(p_mad->attr_id),
+		cl_ntoh32(p_mad_context->cc_context.attr_mod),
+		cl_ntoh64(node_guid), port);
 
 	p_cc_mad = osm_madw_get_cc_mad_ptr(p_madw);
 
@@ -467,8 +470,8 @@ static void cc_rcv_mad(void *context, void *data)
 	p_port = osm_get_port_by_guid(p_cc->subn, port_guid);
 	if (!p_port) {
 		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C109: "
-			"Port guid not in table 0x%" PRIx64 "\n",
-			   port_guid);
+			"Port GUID 0x%" PRIx64 " not in table\n",
+			cl_ntoh64(port_guid));
 		cl_plock_release(&p_osm->lock);
 		goto Exit;
 	}
@@ -507,8 +510,8 @@ static void cc_rcv_mad(void *context, void *data)
 	}
 	else
 		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C10A: "
-			"Unexpected MAD attribute received: %u\n",
-			   p_cc_mad->header.attr_id);
+			"Unexpected MAD attribute ID %u received\n",
+			cl_ntoh16(p_cc_mad->header.attr_id));
 
 	cl_plock_release(&p_osm->lock);
 
@@ -540,7 +543,7 @@ wait:
 	} else
 		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C104: "
 			"send failed to node 0x%" PRIx64 "port %u\n",
-			mad_context.cc_context.node_guid,
+			cl_ntoh64(mad_context.cc_context.node_guid),
 			mad_context.cc_context.port);
 }
 
@@ -674,8 +677,8 @@ static void cc_mad_send_err_callback(void *bind_context,
 	p_port = osm_get_port_by_guid(p_cc->subn, port_guid);
 	if (!p_port) {
 		OSM_LOG(p_cc->log, OSM_LOG_ERROR, "ERR C10B: "
-			"Port guid not in table 0x%" PRIx64 "\n",
-			port_guid);
+			"Port GUID 0x%" PRIx64 " not in table\n",
+			cl_ntoh64(port_guid));
 		cl_plock_release(&p_osm->lock);
 		goto Exit;
 	}
@@ -694,7 +697,7 @@ static void cc_mad_send_err_callback(void *bind_context,
 			ib_get_err_str(p_madw->status),
 			p_madw->p_mad->attr_id,
 			cl_ntoh16(p_madw->mad_addr.dest_lid),
-			node_guid,
+			cl_ntoh64(node_guid),
 			port,
 			cl_ntoh64(p_madw->p_mad->trans_id));
 
