@@ -257,9 +257,6 @@ static void drop_mgr_remove_port(osm_sm_t * sm, IN osm_port_t * p_port)
 		OSM_LOG(sm->p_log, OSM_LOG_VERBOSE,
 			"Cleaned SM for port guid 0x%016" PRIx64 "\n",
 			cl_ntoh64(port_guid));
-		/* clean up the polling_sm pointer */
-		if (sm->p_polling_sm == p_sm)
-			sm->p_polling_sm = NULL;
 		free(p_sm);
 	}
 
@@ -472,7 +469,7 @@ static void drop_mgr_check_node(osm_sm_t * sm, IN osm_node_t * p_node)
 					"Unlinking local node 0x%" PRIx64
 					", port %u"
 					"\n\t\t\t\tand remote node 0x%" PRIx64
-					", port %u\n due to missing PortInfo",
+					", port %u due to missing PortInfo\n",
 					cl_ntoh64(osm_node_get_node_guid
 						  (p_node)), port_num,
 					cl_ntoh64(osm_node_get_node_guid
