@@ -396,11 +396,12 @@ static void del_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 	max_block = (p_port->p_physp->port_info.guid_cap + GUID_TABLE_MAX_ENTRIES - 1) /
 		     GUID_TABLE_MAX_ENTRIES;
 
-	if (block_num > max_block) {
+	if (block_num >= max_block) {
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5116: "
 			"block_num %d is higher than Max GUID Cap block %d "
 			"for port GUID 0x%" PRIx64 "\n",
 			block_num, max_block, cl_ntoh64(p_port->p_physp->port_guid));
+		CL_PLOCK_RELEASE(sa->p_lock);
 		osm_sa_send_error(sa, p_madw,
 				  IB_SA_MAD_STATUS_NO_RECORDS);
 		return;
@@ -417,6 +418,7 @@ static void del_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 	     i++) {
 		/* can't delete block 0 index 0 (base guid is RO) for alias guid table */
 		if (i == 0 && p_sa_mad->comp_mask & IB_GIR_COMPMASK_GID0) {
+			CL_PLOCK_RELEASE(sa->p_lock);
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 				"Not allowed to delete RO GID 0\n");
 			osm_sa_send_error(sa, p_madw,
@@ -436,6 +438,7 @@ static void del_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 				p_list_item = cl_qlist_next(p_list_item);
 				p_mcm_alias_guid = osm_mgrp_get_mcm_alias_guid(p_mcm_port->mgrp, del_alias_guid);
 				if (p_mcm_alias_guid) {
+					CL_PLOCK_RELEASE(sa->p_lock);
 					osm_sa_send_error(sa, p_madw,
 							  IB_SA_MAD_STATUS_DENIED);
 					return;
@@ -481,6 +484,7 @@ static void del_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 	       sizeof(ib_guid_info_t));
 
 Exit:
+	CL_PLOCK_RELEASE(sa->p_lock);
 	gir_respond(sa, p_madw);
 }
 
@@ -499,11 +503,12 @@ static void set_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 
 	max_block = (p_port->p_physp->port_info.guid_cap + GUID_TABLE_MAX_ENTRIES - 1) /
 		     GUID_TABLE_MAX_ENTRIES;
-	if (block_num > max_block) {
+	if (block_num >= max_block) {
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5118: "
 			"block_num %d is higher than Max GUID Cap block %d "
 			"for port GUID 0x%" PRIx64 "\n",
 			block_num, max_block, cl_ntoh64(p_port->p_physp->port_guid));
+		CL_PLOCK_RELEASE(sa->p_lock);
 		osm_sa_send_error(sa, p_madw,
 				  IB_SA_MAD_STATUS_NO_RECORDS);
 		return;
@@ -516,6 +521,7 @@ static void set_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 				"GUID table memory allocation failed for port "
 				"GUID 0x%" PRIx64 "\n",
 				cl_ntoh64(p_port->p_physp->port_guid));
+			CL_PLOCK_RELEASE(sa->p_lock);
 			osm_sa_send_error(sa, p_madw,
 					  IB_SA_MAD_STATUS_NO_RESOURCES);
 			return;
@@ -539,6 +545,7 @@ static void set_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 	     i++) {
 		/* can't set block 0 index 0 (base guid is RO) for alias guid table */
 		if (i == 0 && p_sa_mad->comp_mask & IB_GIR_COMPMASK_GID0) {
+			CL_PLOCK_RELEASE(sa->p_lock);
 			OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
 				"Not allowed to set RO GID 0\n");
 			osm_sa_send_error(sa, p_madw,
@@ -595,6 +602,7 @@ static void set_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 			for (j = 0; j < 1000; j++) {
 				assigned_guid = sm_assigned_guid(sa->p_subn->opt.sm_assigned_guid);
 				if (!assigned_guid) {
+					CL_PLOCK_RELEASE(sa->p_lock);
 					OSM_LOG(sa->p_log, OSM_LOG_ERROR,
 						"ERR 510E: No more assigned guids available\n");
 					osm_sa_send_error(sa, p_madw,
@@ -618,6 +626,7 @@ static void set_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 								OSM_LOG_ERROR,
 								"ERR 510D: osm_assigned_guids_new failed port GUID 0x%" PRIx64 " index %d\n",
 								cl_ntoh64(p_port->p_physp->port_guid), i);
+							CL_PLOCK_RELEASE(sa->p_lock);
 							osm_sa_send_error(sa, p_madw,
 									  IB_SA_MAD_STATUS_NO_RESOURCES);
 							return;
@@ -645,6 +654,7 @@ add_alias_guid:
 				"Alias guid %d memory allocation failed"
 				" for port GUID 0x%" PRIx64 "\n",
 				i, cl_ntoh64(p_port->p_physp->port_guid));
+			CL_PLOCK_RELEASE(sa->p_lock);
 			return;
 		}
 
@@ -702,6 +712,7 @@ add_alias_guid:
 	       &((*p_port->p_physp->p_guids)[block_num * GUID_TABLE_MAX_ENTRIES]),
 	       sizeof(ib_guid_info_t));
 
+	CL_PLOCK_RELEASE(sa->p_lock);
 	gir_respond(sa, p_madw);
 }
 
@@ -725,12 +736,11 @@ static void get_guidinfo(IN osm_sa_t *sa, IN osm_madw_t *p_madw,
 	context.sa = sa;
 	context.p_req_physp = p_req_physp;
 
-	cl_plock_acquire(sa->p_lock);
 
 	cl_qmap_apply_func(&sa->p_subn->node_guid_tbl, sa_gir_by_comp_mask_cb,
 			   &context);
 
-	cl_plock_release(sa->p_lock);
+	CL_PLOCK_RELEASE(sa->p_lock);
 
 	osm_sa_respond(sa, p_madw, sizeof(ib_guidinfo_record_t), &rec_list);
 }
@@ -754,23 +764,25 @@ void osm_gir_rcv_process(IN void *ctx, IN void *data)
 
 	CL_ASSERT(p_rcvd_mad->attr_id == IB_MAD_ATTR_GUIDINFO_RECORD);
 
-	/* update the requester physical port */
-	p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
-						osm_madw_get_mad_addr_ptr(p_madw));
-	if (p_req_physp == NULL) {
-		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5104: "
-			"Cannot find requester physical port\n");
-		goto Exit;
-	}
-	OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
-		"Requester port GUID 0x%" PRIx64 "\n",
-		cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
-
 	switch(p_rcvd_mad->method) {
 	case IB_MAD_METHOD_GET:
 	case IB_MAD_METHOD_GETTABLE:
+		/* update the requester physical port */
+		CL_PLOCK_ACQUIRE(sa->p_lock);
+		p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
+							osm_madw_get_mad_addr_ptr(p_madw));
+		if (p_req_physp == NULL) {
+			CL_PLOCK_RELEASE(sa->p_lock);
+			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5104: "
+				"Cannot find requester physical port\n");
+			goto Exit;
+		}
+		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+			"Requester port GUID 0x%" PRIx64 "\n",
+			cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
+
 		get_guidinfo(sa, p_madw, p_req_physp);
-		break;
+		goto Exit;
 	case IB_MAD_METHOD_SET:
 	case IB_MAD_METHOD_DELETE:
 		if (!check_mod_comp_mask(p_rcvd_mad->comp_mask)) {
@@ -784,23 +796,40 @@ void osm_gir_rcv_process(IN void *ctx, IN void *data)
 			goto Exit;
 		}
 		p_rcvd_rec = (ib_guidinfo_record_t *) ib_sa_mad_get_payload_ptr(p_rcvd_mad);
+
+		/* update the requester physical port */
+		CL_PLOCK_EXCL_ACQUIRE(sa->p_lock);
+		p_req_physp = osm_get_physp_by_mad_addr(sa->p_log, sa->p_subn,
+							osm_madw_get_mad_addr_ptr(p_madw));
+		if (p_req_physp == NULL) {
+			CL_PLOCK_RELEASE(sa->p_lock);
+			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5104: "
+				"Cannot find requester physical port\n");
+			goto Exit;
+		}
+		OSM_LOG(sa->p_log, OSM_LOG_DEBUG,
+			"Requester port GUID 0x%" PRIx64 "\n",
+			cl_ntoh64(osm_physp_get_port_guid(p_req_physp)));
+
 		p_port = osm_get_port_by_lid(sa->p_subn, p_rcvd_rec->lid);
 		if (!p_port) {
+			CL_PLOCK_RELEASE(sa->p_lock);
 			OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5117: "
 				"Port with LID %u not found\n",
 				cl_ntoh16(p_rcvd_rec->lid));
-			osm_sa_send_error(sa, p_madw, IB_SA_MAD_STATUS_NO_RECORDS);
+				osm_sa_send_error(sa, p_madw, IB_SA_MAD_STATUS_NO_RECORDS);
 			goto Exit;
 		}
 		if (!osm_physp_share_pkey(sa->p_log, p_req_physp, p_port->p_physp,
-					  sa->p_subn->opt.allow_both_pkeys))
+					  sa->p_subn->opt.allow_both_pkeys)) {
+			CL_PLOCK_RELEASE(sa->p_lock);
 			goto Exit;
-		CL_PLOCK_EXCL_ACQUIRE(sa->p_lock);
+		}
+
 		if (p_rcvd_mad->method == IB_MAD_METHOD_SET)
 			set_guidinfo(sa, p_madw, p_port, p_rcvd_rec->block_num);
 		else
 			del_guidinfo(sa, p_madw, p_port, p_rcvd_rec->block_num);
-		CL_PLOCK_RELEASE(sa->p_lock);
 		break;
 	default:
 		OSM_LOG(sa->p_log, OSM_LOG_ERROR, "ERR 5105: "
