@@ -392,11 +392,16 @@ static void smi_rcv_process_get_sm(IN osm_sm_t * sm,
 			 * as it might not get it and we don't want to wait for a HANDOVER
 			 * forever.
 			 */
-			if (sm->p_polling_sm) {
+			if (sm->polling_sm_guid) {
 				if (smi_rcv_remote_sm_is_higher(sm, p_smi))
 					sm->p_subn->force_heavy_sweep = TRUE;
-				else
+				else {
+					/* Update master_sm_guid to the GUID of the newly
+					 * found MASTER SM and send trap 144 to it.
+					 */
+					sm->master_sm_guid = sm->polling_sm_guid;
 					osm_send_trap144(sm, TRAP_144_MASK_SM_PRIORITY_CHANGE);
+				}
 				osm_sm_state_mgr_signal_master_is_alive(sm);
 			} else {
 				/* This is a response we got while sweeping the subnet.
@@ -540,6 +545,12 @@ static void smi_rcv_process_set_response(IN osm_sm_t * sm,
 	CL_ASSERT(p_madw);
 
 	p_smp = osm_madw_get_smp_ptr(p_madw);
+	if (ib_smp_get_status(p_smp)) {
+		OSM_LOG(sm->p_log, OSM_LOG_DEBUG,
+			"MAD status 0x%x received\n",
+			cl_ntoh16(ib_smp_get_status(p_smp)));
+		goto Exit;
+	}
 
 	if (p_smp->method != IB_MAD_METHOD_GET_RESP) {
 		OSM_LOG(sm->p_log, OSM_LOG_ERROR, "ERR 2F16: "
